import pathlib

from honeybee_ies.writer import model_to_ies as hb_model_to_ies
from dragonfly.model import Model


def model_to_ies(
        model: Model, folder: str = '.', name: str = None, enforce_adj: bool = True
    ) -> pathlib.Path:
    """Export a dragonfly model to an IES GEM file.

    Args:
        model: A dragonfly model.
        folder: Path to target folder to export the file. Default is current folder.
        name: An optional name for exported file. By default the name of the model will
            be used.
        enforce_adj: Boolean to note whether an exception should be raised if
            an adjacency between two Room2Ds is invalid (True) or if the invalid
            Surface boundary condition should be replaced with an Outdoor
            boundary condition (False). (Default: True).

    Returns:
        Path to exported GEM file.
    """
    hb_models = model.to_honeybee(
        object_per_model='District', use_multiplier=False,
        solve_ceiling_adjacencies=False, enforce_adj=enforce_adj
    )
    return hb_model_to_ies(hb_models[0], folder, name)



import pathlib
from dragonfly_ies.writer import model_to_ies
from dragonfly.model import Model


def test_df_model():
    in_file = './tests/assets/university_campus.dfjson'
    out_folder = pathlib.Path('./tests/assets/temp')
    out_folder.mkdir(parents=True, exist_ok=True) 
    model = Model.from_file(in_file)
    outf = model_to_ies(model, out_folder.as_posix(), name='uni_model')
    assert outf.exists()



"""dragonfly-ies commands which will be added to dragonfly command line interface."""
import click
from dragonfly.cli import main

from .translate import translate


# command group for all ies extension commands.
@click.group(help='dragonfly ies commands.')
@click.version_option()
def ies():
    pass


ies.add_command(translate)

# add ies sub-commands to honeybee CLI
main.add_command(ies)



import os

from click.testing import CliRunner

from ladybug.futil import nukedir

from dragonfly_ies.cli.translate import model_to_gem


def test_df_model_to_ies():
    runner = CliRunner()
    input_df_model = './tests/assets/simple_model.dfjson'
    folder = './tests/assets/temp'
    name = 'cli_df_model'

    result = runner.invoke(
        model_to_gem, [input_df_model, '--folder', folder, '--name', name]
    )
    assert result.exit_code == 0
    assert os.path.isfile(os.path.join(folder, f'{name}.gem'))
    nukedir(folder, True)


def test_invalid_df_model_to_ies():
    runner = CliRunner()
    input_df_model = './tests/assets/model_invalid_adj.dfjson'
    folder = './tests/assets/temp'
    name = 'cli_invalid_df_model'

    result = runner.invoke(
        model_to_gem,
        [input_df_model, '--folder', folder, '--name', name, '--bypass-adj-check']
    )
    assert result.exit_code == 0
    assert os.path.isfile(os.path.join(folder, f'{name}.gem'))
    nukedir(folder, True)



"""honeybee ies translation commands."""
import click
import sys
import pathlib
import logging

from dragonfly.model import Model

_logger = logging.getLogger(__name__)


@click.group(help='Commands for translating Dragonfly JSON files to IES files.')
def translate():
    pass


@translate.command('model-to-gem')
@click.argument('model-json', type=click.Path(
    exists=True, file_okay=True, dir_okay=False, resolve_path=True))
@click.option(
    '--name', '-n', help='Name of the output file.', default="model", show_default=True
)
@click.option(
    '--folder', '-f', help='Path to target folder.',
    type=click.Path(exists=False, file_okay=False, resolve_path=True,
                    dir_okay=True), default='.', show_default=True
)
@click.option('--enforce-adj-check/--bypass-adj-check', ' /-bc', help='Flag to note '
              'whether an exception should be raised if an adjacency between two '
              'Room2Ds is invalid or if the check should be bypassed and the invalid '
              'Surface boundary condition should be replaced with an Outdoor boundary '
              'condition.', default=True, show_default=True)
def model_to_gem(model_json, name, folder, enforce_adj_check):
    """Translate a Dragonfly Model JSON file to an IES GEM file.

    \b
    Args:
        model_json: Full path to a Model JSON file (DFJSON) or a Model pkl (DFpkl) file.

    """
    try:
        model = Model.from_file(model_json)
        folder = pathlib.Path(folder)
        folder.mkdir(parents=True, exist_ok=True)
        model.to_gem(folder.as_posix(), name=name, enforce_adj=enforce_adj_check)
    except Exception as e:
        _logger.exception('Model translation failed.\n{}'.format(e))
        sys.exit(1)
    else:
        sys.exit(0)
